@@ -1,12 +1,12 @@
 import re
-from functools import partial
+from functools import partial, reduce
 
 from content_editor.models import Type
 from django import forms
 from django.core import validators
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
 from django.db import models
-from django.db.models import signals
+from django.db.models import F, Value, signals
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.template.defaultfilters import truncatechars
 from django.utils.crypto import get_random_string
@@ -82,8 +82,7 @@ class NameField(models.CharField):
 
 class FormFieldBase(models.Model):
     """
-    Form field plugins must inherit this model if using the
-    ``get_formfields_union`` helper for configured form validation.
+    Form field plugins must inherit this model
     """
 
     name = NameField()
@@ -128,6 +127,14 @@ class FormFieldBase(models.Model):
             f"{self._meta.label_lower} needs a get_loaders implementation"
         )
 
+    @staticmethod
+    def set_field_type(sender, **kwargs):
+        if issubclass(sender, FormFieldBase) and not sender._meta.abstract:
+            sender.type = sender.__name__.lower()
+
+
+signals.class_prepared.connect(FormFieldBase.set_field_type)
+
 
 class ConfiguredForm(models.Model):
     name = models.CharField(_("name"), max_length=1000)
@@ -159,22 +166,41 @@ class ConfiguredForm(models.Model):
             types = {type.key: type for type in sender.FORMS}
             sender.type = property(lambda self: types.get(self.form_type))
 
-    def get_formfields_union(
-        self, *, plugins, values=None, field_plugin_bases=(FormFieldBase,)
-    ):
-        qs = None
-        values = ["name"] if values is None else values
+    def get_formfields_union(self, *, plugins, attributes=None):
+        values = ["name"]
+        columns = []
+        for index, attribute in enumerate(attributes or []):
+            alias = f"__val_{index}"
+            values.append(alias)
+            columns.append((alias, attribute))
+
+        querysets = []
         for plugin in plugins:
-            if not issubclass(plugin, field_plugin_bases):
+            if not issubclass(plugin, FormFieldBase):
                 continue
-            plugin_qs = plugin.objects.filter(parent=self).values_list(
-                *values, flat=len(values) == 1
-            )
-            if qs is None:
-                qs = plugin_qs
-            else:
-                qs = qs.union(plugin_qs, all=True)
-        return qs
+            qs = plugin.objects.filter(parent=self)
+            annotations = {}
+            for alias, attribute in columns:
+                # See https://code.djangoproject.com/ticket/28553
+                # If we could rely on values_list returning columns in the
+                # specified order **for all querysets** we wouldn't have to do
+                # this. But since that isn't the case we use .annotate() for
+                # all values, even those which 1:1 exist as a column in the
+                # database. I'm not sure if the enumeration is necessary but it
+                # certainly doesn't hurt (more).
+                try:
+                    plugin._meta.get_field(attribute)
+                except FieldDoesNotExist:
+                    annotations[alias] = Value(getattr(plugin, attribute, ""))
+                else:
+                    annotations[alias] = F(attribute)
+            qs = qs.annotate(**annotations)
+            querysets.append(qs.values_list(*values))
+        qs = reduce(lambda p, q: p.union(q, all=True), querysets[1:], querysets[0])
+        return [
+            (row[0], {column[1]: value for column, value in zip(columns, row[1:])})
+            for row in qs
+        ]
 
 
 signals.class_prepared.connect(ConfiguredForm.fill_form_choices)
@@ -262,13 +288,15 @@ class SimpleFieldBase(FormField):
 
         meta_class = type("Meta", (cls.Meta,), meta)
 
-        cls = type(
+        return type(
             f"{cls.__qualname__}_{type_name}",
             (cls,),
-            {"__module__": cls.__module__, "Meta": meta_class},
+            {
+                "__module__": cls.__module__,
+                "Meta": meta_class,
+                "TYPE": type_name,
+            },
         )
-        cls.TYPE = type_name
-        return cls
 
     def clean_fields(self, exclude=None):
         super().clean_fields(exclude)
